module SedmaGamble where

import Prelude
import SedmaBase
import SedmaReplay

select A (a:_:_:_:_) = a
select B (_:b:_:_:_) = b
select C (_:_:c:_:_) = c
select D (_:_:_:d:_) = d

data GlobalState s = GlobalState {
   deck :: Cards,
   leader :: Player,
   hands :: [Hand],
   states :: [s]
}

order :: Player -> Cards -> Cards
order A [c1,c2,c3,c4] = [c1,c2,c3,c4]
order B [c1,c2,c3,c4] = [c4,c1,c2,c3]
order C [c1,c2,c3,c4] = [c3,c4,c1,c2]
order D [c1,c2,c3,c4] = [c2,c3,c4,c1]

refill :: Cards -> [Hand] -> Player -> (Cards, [Hand])
refill []    hands _      = ([], hands)
refill cards hands leader = (cards', hands') where
   (dealed, cards') = splitAt 4 cards
   ordered = order leader dealed
   hands' = map (uncurry (:)) (zip ordered hands)

playRound :: PlayerState s => GlobalState s -> [AIPlayer s] -> Trick
playRound (GlobalState deck leader hands states) ais = trick where
   order = take 4 $ dropWhile (/= leader) players
   playCard p t = (select p ais) (select p hands) t (select p states)
   trick = foldl (\ t p -> t++[playCard p t]) [] order

playGame :: PlayerState s => GlobalState s -> [AIPlayer s] -> Cards -> Cards
playGame (GlobalState [] _ [[],[],[],[]] _) _ cards = cards
playGame gs ais cards = playGame gs' ais (cards++trick) where
   trick = playRound gs ais
   leader' = winner trick (leader gs)
   states' = map (updateState trick (leader gs)) (states gs)
   hands' = map (filter ((flip notElem) trick)) (hands gs)
   (deck', hands'') = refill (deck gs) hands' leader'
   gs' = GlobalState deck' leader' hands'' states'

gamble :: PlayerState s => [AIPlayer s] -> Cards -> Cards
gamble ais cards = playGame gs ais [] where
   hands = [[],[],[],[]]
   (cards',hands') = iter 4 (\ (cs,hs) -> refill cs hs A) (cards,hands)
   states = map initState (take 4 players)
   gs = GlobalState cards' A hands' states

